#!/usr/bin/python
# -*- coding: utf-8 -*-
# MIT License
#
# Copyright (c) 2022 SIANA Systems
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Upgrade script.

:date:      2022
:author:    SIANA Systems
:contact:   support@siana-systems.com
:license:   The MIT License (MIT)
"""
# --------------------------------------
__author__ = "SIANA Systems"
__version__ = "ALPHA"
# -------------------------------------

import os
import subprocess as sp


# -->> Tunables <<---------------------
APP_PARTITION_ADDRESS = 0x08000000


# -->> Definitions <<------------------
platform = os.name

# Platform dependence
stm_dirs = ["/Applications/STMicroelectronics",
            "C:/Program Files/STMicroelectronics"]

programmer_dirs = ["/Applications/STMicroelectronics/STM32Cube/STM32CubeProgrammer/STM32CubeProgrammer.app/Contents/MacOs/bin",
                   "C:/Program Files/STMicroelectronics/STM32Cube/STM32CubeProgrammer/bin"]

programmer_exes = ["STM32_Programmer_CLI",
                   "STM32_Programmer_CLI.exe"]

stm_dir = stm_dirs[platform == 'nt']
programmer_dir = programmer_dirs[platform == 'nt']
programmer_exe = programmer_exes[platform == 'nt']

bin_ext = ".bin"
file_marker = "F150"

# -->> API <<--------------------------
def script() -> None:
    """Script main logic"""

    for (root, dir, files) in os.walk(os.path.expandvars(stm_dir)):
        if programmer_exe in files:
            programmer_dir = root
            break

    programmer_cmd = os.path.expandvars(os.path.join(programmer_dir, programmer_exe))
    print("Verifying " + programmer_cmd + " exists...")

    if not os.path.exists(programmer_cmd):
        print("\n********************ERROR********************")
        print("  Cannot locate STM32CubeProgrammer executable")

    else:
        cmd = [programmer_cmd, '-c', 'port=usb1', '-w' ]
        print("looking for files")

        local_dir = os.getcwd()
        bin_dir = local_dir+'/.'
        root, dirs, files = next(os.walk(bin_dir))
        bincount = 0
        app_file = None
        for file in files:
            print(file)
            name, ext = os.path.splitext(file)
            if file_marker in name and bin_ext in ext:
                bincount += 1
                if app_file is None:
                    print ("Using app file = " + file)
                    app_file  = file

    if (app_file is None):
        print("\n***********ERROR***********")
        print("  Cannot locate binary files")
    else:
        # Execute
        sp.call(cmd + [app_file, f"0x{APP_PARTITION_ADDRESS:08X}"])
        print("")
        print("**********************************************************************************")
        print("You need to power cycle the device by pressing the button for more than 10 seconds")
        print("**********************************************************************************")
    
    input("\nPress enter to close")


# -->> Execute <<----------------------
if __name__ == '__main__':
    script()